

// #pragma once is supported starting with _MCS_VER 1000, 
// so we need not to check the version (because we only support _MSC_VER >= 1100)!
#pragma once

#include <windows.h>

#ifdef __cplusplus
extern "C" {
#endif

HRESULT InitLeakFinder();
void DeinitLeakFinder();

#ifdef __cplusplus
}
#endif


// The following is only available if the file is CPP
#ifdef __cplusplus

#include "StackWalker.h"

// Interface for output...
class LeakFinderOutput : public StackWalker
{
public:
  typedef enum LeakFinderOptions
  {
    // No addition info will be retrived 
    // (only the address is available)
    LeakFinderNone = 0,
    LeakFinderShowCompleteCallstack = 0x1000
  } LeakFinderOptions;

  LeakFinderOutput(int options = OptionsAll,  LPCSTR szSymPath = NULL);
  virtual void OnLeakSearchStart(LPCSTR sszLeakFinderName);
  virtual void OnLeakStartEntry(LPCSTR szKeyName, SIZE_T nDataSize);
protected:
  virtual void OnCallstackEntry(CallstackEntryType eType, CallstackEntry &entry);
  virtual void OnOutput(LPCSTR szText) { printf(szText); StackWalker::OnOutput(szText); }
  virtual void OnDbgHelpErr(LPCSTR szFuncName, DWORD gle, DWORD64 addr)
  {
    if (strcmp(szFuncName, "SymGetLineFromAddr64") == 0) return;
    StackWalker::OnDbgHelpErr(szFuncName, gle, addr);
  }
};

// C++ interface:
void DeinitLeakFinder(LeakFinderOutput *output);

class ZZZ_LeakFinder
{
public:
  ZZZ_LeakFinder()
  {
    InitLeakFinder();
  }
  ~ZZZ_LeakFinder()
  {
    DeinitLeakFinder();
  }
};

#if defined(INIT_LEAK_FINDER)
#pragma warning(push)
#pragma warning (disable:4074)
#pragma init_seg (compiler)
ZZZ_LeakFinder zzz_LeakFinder;
#pragma warning(pop)
#endif

#endif  // __cplusplus
